(function() {
  var Client, Me, Organization, Repository, User, request;

  request = require('request');

  Me = require('./me');

  User = require('./user');

  Repository = require('./repository');

  Organization = require('./organization');

  Client = (function() {

    function Client(token) {
      this.token = token;
    }

    Client.prototype.me = function() {
      return new Me(this);
    };

    Client.prototype.user = function(name) {
      return new User(name, this);
    };

    Client.prototype.repository = function(name) {
      return new Repository(name, this);
    };

    Client.prototype.organization = function(name) {
      return new Organization(name, this);
    };

    Client.prototype.query = function(path) {
      var uri;
      if (path == null) path = '/';
      if (path[0] !== '/') path = '/' + path;
      uri = "https://api.github.com" + path;
      return uri += this.token ? "?access_token=" + this.token : '';
    };

    Client.prototype.errorHandle = function(res, body, callback) {
      var _ref;
      if (res.statusCode === 422) return callback(new Error(body.message));
      if ((_ref = res.statusCode) === 400 || _ref === 401 || _ref === 404) {
        return callback(new Error(body.message));
      }
      return callback(null, res.statusCode, body);
    };

    Client.prototype.get = function(path, callback) {
      var _this = this;
      return request({
        uri: this.query(path),
        method: 'GET'
      }, function(err, res, body) {
        if (err) return callback(err);
        return _this.errorHandle(res, JSON.parse(body), callback);
      });
    };

    Client.prototype.post = function(path, content, callback) {
      var _this = this;
      if (content == null) content = {};
      return request({
        uri: this.query(path),
        method: 'POST',
        body: JSON.stringify(content),
        headers: {
          'Content-Type': 'application/json'
        }
      }, function(err, res, body) {
        if (err) return callback(err);
        return _this.errorHandle(res, JSON.parse(body), callback);
      });
    };

    Client.prototype.put = function(path, content, callback) {
      var _this = this;
      if (content == null) content = {};
      return request({
        uri: this.query(path),
        method: 'PUT',
        body: JSON.stringify(content),
        headers: {
          'Content-Type': 'application/json'
        }
      }, function(err, res, body) {
        if (err) return callback(err);
        return _this.errorHandle(res, JSON.parse(body), callback);
      });
    };

    Client.prototype.del = function(path, content, callback) {
      var _this = this;
      if (content == null) content = {};
      return request({
        uri: this.query(path),
        method: 'DELETE',
        body: JSON.stringify(content),
        headers: {
          'Content-Type': 'application/json'
        }
      }, function(err, res, body) {
        if (err) return callback(err);
        return _this.errorHandle(res, JSON.parse(body), callback);
      });
    };

    return Client;

  })();

  module.exports = Client;

}).call(this);
